import param
import pytest

from panel.io.location import Location
from panel.util import edit_readonly


@pytest.fixture
def location():
    loc = Location()
    with edit_readonly(loc):
        loc.href = "http://localhost:5006"
        loc.hostname = "localhost"
        loc.pathname = ""
        loc.protocol = 'http'
        loc.search = ""
        loc.hash = ""
    return loc


class SyncParameterized(param.Parameterized):

    integer = param.Integer(default=None)

    string = param.String(default=None)


def test_location_update_query(location):
    location.update_query(a=1)
    assert location.search == "?a=1"
    location.update_query(b='c')
    assert location.search == "?a=1&b=c"

def test_location_sync_query_init(location):
    p = SyncParameterized(integer=1, string='abc')
    location.sync(p)
    assert location.search == "?integer=1&string=abc"
    location.unsync(p)
    assert location._synced == []
    assert location.search == ""

def test_location_unsync(location):
    p = SyncParameterized(integer=1, string='abc')
    location.sync(p)
    assert location.search == "?integer=1&string=abc"
    location.unsync(p)
    assert location.search == ""
    location.update_query(integer=2, string='def')
    assert p.integer == 1
    assert p.string == "abc"
    p.integer = 3
    p.string = "ghi"
    assert location.search == "?integer=2&string=def"

def test_location_unsync_partial(location):
    p = SyncParameterized(integer=1, string='abc')
    location.sync(p)
    assert location.search == "?integer=1&string=abc"
    location.unsync(p, ['string'])
    assert location.search == "?integer=1"
    location.update_query(integer=2, string='def')
    assert p.integer == 2
    assert p.string == "abc"
    p.integer = 3
    p.string = "ghi"
    assert location.search == "?integer=3&string=def"

def test_location_sync_query_init_partial(location):
    p = SyncParameterized(integer=1, string='abc')
    location.sync(p, ['integer'])
    assert location.search == "?integer=1"
    location.unsync(p)
    assert location._synced == []

def test_location_sync_query_init_rename(location):
    p = SyncParameterized(integer=1, string='abc')
    location.sync(p, {'integer': 'int', 'string': 'str'})
    assert location.search == "?int=1&str=abc"
    location.unsync(p)
    assert location._synced == []
    assert location.search == ""

def test_location_sync_query(location):
    p = SyncParameterized()
    location.sync(p)
    p.integer = 2
    assert location.search == "?integer=2"
    location.unsync(p)
    assert location._synced == []
    assert location.search == ""

def test_location_sync_param_init(location):
    p = SyncParameterized()
    location.search = "?integer=1&string=abc"
    location.sync(p)
    assert p.integer == 1
    assert p.string == "abc"
    location.unsync(p)
    assert location._synced == []
    assert location.search == ""

def test_location_sync_on_error(location):
    p = SyncParameterized(string='abc')
    changes = []
    def on_error(change):
        changes.append(change)
    location.sync(p, on_error=on_error)
    location.search = "?integer=a&string=abc"
    assert changes == [{'integer': 'a'}]
    location.unsync(p)
    assert location._synced == []
    assert location.search == ""

def test_location_sync_param_init_partial(location):
    p = SyncParameterized()
    location.search = "?integer=1&string=abc"
    location.sync(p, ['integer'])
    assert p.integer == 1
    assert p.string is None
    location.unsync(p)
    assert location._synced == []
    assert location.search == "?string=abc"

def test_location_sync_param_init_rename(location):
    p = SyncParameterized()
    location.search = "?int=1&str=abc"
    location.sync(p, {'integer': 'int', 'string': 'str'})
    assert p.integer == 1
    assert p.string == 'abc'
    location.unsync(p)
    assert location._synced == []
    assert location.search == ""

def test_location_sync_param_update(location):
    p = SyncParameterized()
    location.sync(p)
    location.search = "?integer=1&string=abc"
    assert p.integer == 1
    assert p.string == "abc"
    location.unsync(p)
    assert location._synced == []
    assert location.search == ""



import io
import sys
import tempfile
import time

import param
import pytest

from panel.io.server import serve
from panel.layout import Column, Tabs
from panel.widgets import FileDownload, TextInput

pytestmark = pytest.mark.ui
not_windows = pytest.mark.skipif(sys.platform=='win32', reason="Does not work on Windows")


@not_windows
def test_file_download_updates_when_navigating_between_dynamic_tabs(page, port):
    text_input = TextInput(value='abc')

    @param.depends(text_input.param.value)
    def create_file(value):
        return io.StringIO(value)

    download = FileDownload(
        callback=create_file, filename='f.txt', embed=False
    )

    tabs = Tabs(
        ("Download", Column(text_input, download)),
        ("Dummy", "dummy tab"),
        dynamic=True
    )

    serve(tabs, port=port, threaded=True, show=False)

    time.sleep(0.5)

    page.goto(f"http://localhost:{port}")

    with page.expect_download() as download_info:
        page.click('.bk-btn > a')

    download = download_info.value
    tmp = tempfile.NamedTemporaryFile(suffix='.txt')
    download.save_as(tmp.name)
    assert tmp.file.read().decode('utf-8') == 'abc'

    page.click('.bk-tab:not(.bk-active)')
    page.click('.bk-tab:not(.bk-active)')

    page.click('.bk-input')

    page.keyboard.type('def')
    page.keyboard.press('Enter')

    with page.expect_download() as download_info:
        page.click('.bk-btn > a')

    download = download_info.value
    tmp = tempfile.NamedTemporaryFile(suffix='.txt')
    download.save_as(tmp.name)
    assert tmp.file.read().decode('utf-8') == 'abcdef'



from panel.layout import Spacer, Swipe


def test_swipe_construct(document, comm):
    before = Spacer(styles={"background": "red"}, height=400, width=800)
    after = Spacer(styles={"background": "green"}, height=400, width=800)
    swipe = Swipe(
        before,
        after,
        value=20,
        height=800,
        slider_width=15,
        slider_color="lightgray"
    )

    model = swipe.get_root(document, comm)

    assert model.children == {
        'before-inner': [before._models[model.ref['id']][0]],
        'after-inner': [after._models[model.ref['id']][0]]
    }
